@@ -7,11 +7,11 @@ pub mod batch;
 pub mod prepared_statement;
 pub mod query;
 
-pub use crate::frame::types::Consistency;
+pub use crate::frame::types::{Consistency, SerialConsistency};
 
 pub struct StatementConfig {
-    pub consistency: Consistency,
-    pub serial_consistency: Option<Consistency>,
+    pub consistency: Option<Consistency>,
+    pub serial_consistency: Option<SerialConsistency>,
 
     pub is_idempotent: bool,
 
@@ -19,17 +19,19 @@ pub struct StatementConfig {
     pub speculative_execution_policy: Option<Arc<dyn SpeculativeExecutionPolicy>>,
 
     pub tracing: bool,
+    pub timestamp: Option<i64>,
 }
 
 impl Default for StatementConfig {
     fn default() -> Self {
         Self {
             consistency: Default::default(),
-            serial_consistency: None,
+            serial_consistency: Some(SerialConsistency::LocalSerial),
             is_idempotent: false,
             retry_policy: None,
             speculative_execution_policy: None,
             tracing: false,
+            timestamp: None,
         }
     }
 }
@@ -46,6 +48,14 @@ impl Clone for StatementConfig {
                 .map(|policy| policy.clone_boxed()),
             speculative_execution_policy: self.speculative_execution_policy.clone(),
             tracing: self.tracing,
+            timestamp: self.timestamp,
         }
     }
 }
+
+impl StatementConfig {
+    /// Determines the consistency of a query
+    pub fn determine_consistency(&self, default_consistency: Consistency) -> Consistency {
+        self.consistency.unwrap_or(default_consistency)
+    }
+}
