@@ -1,14 +1,16 @@
 use crate::frame::response::event::Event;
 use crate::routing::Token;
 use crate::transport::connection::{Connection, ConnectionConfig};
-use crate::transport::connection_keeper::ConnectionKeeper;
+use crate::transport::connection_pool::{NodeConnectionPool, PoolConfig, PoolSize};
 use crate::transport::errors::QueryError;
 use crate::transport::session::IntoTypedRows;
 
+use crate::statement::query::Query;
 use rand::seq::SliceRandom;
 use rand::thread_rng;
 use std::collections::HashMap;
 use std::net::{IpAddr, SocketAddr};
+use std::num::NonZeroUsize;
 use std::str::FromStr;
 use tokio::sync::mpsc;
 use tracing::{debug, error, warn};
@@ -17,7 +19,7 @@ use tracing::{debug, error, warn};
 pub struct TopologyReader {
     connection_config: ConnectionConfig,
     control_connection_address: SocketAddr,
-    control_connection: ConnectionKeeper,
+    control_connection: NodeConnectionPool,
 
     // when control connection fails, TopologyReader tries to connect to one of known_peers
     known_peers: Vec<SocketAddr>,
@@ -42,6 +44,7 @@ pub struct Keyspace {
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
+#[allow(clippy::enum_variant_names)]
 pub enum Strategy {
     SimpleStrategy {
         replication_factor: usize,
@@ -73,12 +76,9 @@ impl TopologyReader {
         // - send received events via server_event_sender
         connection_config.event_sender = Some(server_event_sender);
 
-        let control_connection = ConnectionKeeper::new(
+        let control_connection = Self::make_control_connection_pool(
             control_connection_address,
             connection_config.clone(),
-            None,
-            None,
-            None,
         );
 
         TopologyReader {
@@ -121,12 +121,9 @@ impl TopologyReader {
             );
 
             self.control_connection_address = *peer;
-            self.control_connection = ConnectionKeeper::new(
+            self.control_connection = Self::make_control_connection_pool(
                 self.control_connection_address,
                 self.connection_config.clone(),
-                None,
-                None,
-                None,
             );
 
             result = self.fetch_topology_info().await;
@@ -163,13 +160,32 @@ impl TopologyReader {
             .map(|peer| peer.address)
             .collect();
     }
+
+    fn make_control_connection_pool(
+        addr: SocketAddr,
+        connection_config: ConnectionConfig,
+    ) -> NodeConnectionPool {
+        let pool_config = PoolConfig {
+            connection_config,
+
+            // We want to have only one connection to receive events from
+            pool_size: PoolSize::PerHost(NonZeroUsize::new(1).unwrap()),
+
+            // The shard-aware port won't be used with PerHost pool size anyway,
+            // so explicitly disable it here
+            can_use_shard_aware_port: false,
+        };
+
+        NodeConnectionPool::new(addr.ip(), addr.port(), pool_config, None)
+    }
 }
 
 async fn query_topology_info(
-    conn_keeper: &ConnectionKeeper,
+    pool: &NodeConnectionPool,
     connect_port: u16,
 ) -> Result<TopologyInfo, QueryError> {
-    let conn: &Connection = &*conn_keeper.get_connection().await?;
+    pool.wait_until_initialized().await;
+    let conn: &Connection = &*pool.random_connection()?;
 
     let peers_query = query_peers(conn, connect_port);
     let keyspaces_query = query_keyspaces(conn);
@@ -194,17 +210,16 @@ async fn query_topology_info(
 }
 
 async fn query_peers(conn: &Connection, connect_port: u16) -> Result<Vec<Peer>, QueryError> {
-    // There shouldn't be more peers than a single page capacity
-    let peers_query = conn.query_single_page(
-        "select peer, data_center, rack, tokens from system.peers",
-        &[],
-    );
-    let local_query = conn.query_single_page(
-        "select rpc_address, data_center, rack, tokens from system.local",
-        &[],
-    );
+    let mut peers_query = Query::new("select peer, data_center, rack, tokens from system.peers");
+    peers_query.set_page_size(1024);
+    let peers_query_future = conn.query_all(&peers_query, &[]);
 
-    let (peers_res, local_res) = tokio::try_join!(peers_query, local_query)?;
+    let mut local_query =
+        Query::new("select rpc_address, data_center, rack, tokens from system.local");
+    local_query.set_page_size(1024);
+    let local_query_future = conn.query_all(&local_query, &[]);
+
+    let (peers_res, local_res) = tokio::try_join!(peers_query_future, local_query_future)?;
 
     let peers_rows = peers_res.rows.ok_or(QueryError::ProtocolError(
         "system.peers query response was not Rows",
@@ -238,7 +253,7 @@ async fn query_peers(conn: &Connection, connect_port: u16) -> Result<Vec<Peer>,
         // Parse string representation of tokens as integer values
         let tokens: Vec<Token> = tokens_str
             .iter()
-            .map(|s| Token::from_str(&s))
+            .map(|s| Token::from_str(s))
             .collect::<Result<Vec<Token>, _>>()
             .map_err(|_| QueryError::ProtocolError("Couldn't parse tokens as integer values"))?;
 
@@ -254,26 +269,25 @@ async fn query_peers(conn: &Connection, connect_port: u16) -> Result<Vec<Peer>,
 }
 
 async fn query_keyspaces(conn: &Connection) -> Result<HashMap<String, Keyspace>, QueryError> {
-    let rows = conn
-        .query_single_page(
-            "select keyspace_name, toJson(replication) from system_schema.keyspaces",
-            &[],
-        )
-        .await?
-        .rows
-        .ok_or(QueryError::ProtocolError(
-            "system_schema.keyspaces query response was not Rows",
-        ))?;
+    let mut keyspaces_query =
+        Query::new("select keyspace_name, replication from system_schema.keyspaces");
+    keyspaces_query.set_page_size(1024);
+
+    let rows =
+        conn.query_all(&keyspaces_query, &[])
+            .await?
+            .rows
+            .ok_or(QueryError::ProtocolError(
+                "system_schema.keyspaces query response was not Rows",
+            ))?;
 
     let mut result = HashMap::with_capacity(rows.len());
 
-    for row in rows.into_typed::<(String, String)>() {
-        let (keyspace_name, keyspace_json_text) = row.map_err(|_| {
+    for row in rows.into_typed::<(String, HashMap<String, String>)>() {
+        let (keyspace_name, strategy_map) = row.map_err(|_| {
             QueryError::ProtocolError("system_schema.keyspaces has invalid column type")
         })?;
 
-        let strategy_map: HashMap<String, String> = json_to_string_map(&keyspace_json_text)?;
-
         let strategy: Strategy = strategy_from_string_map(strategy_map)?;
 
         result.insert(keyspace_name, Keyspace { strategy });
@@ -282,37 +296,6 @@ async fn query_keyspaces(conn: &Connection) -> Result<HashMap<String, Keyspace>,
     Ok(result)
 }
 
-fn json_to_string_map(json_text: &str) -> Result<HashMap<String, String>, QueryError> {
-    use serde_json::Value;
-
-    let json: Value = serde_json::from_str(json_text)
-        .map_err(|_| QueryError::ProtocolError("Couldn't parse keyspaces as json"))?;
-
-    let object_map = match json {
-        Value::Object(map) => map,
-        _ => {
-            return Err(QueryError::ProtocolError(
-                "keyspaces map json is not a json object",
-            ))
-        }
-    };
-
-    let mut result = HashMap::with_capacity(object_map.len());
-
-    for (key, val) in object_map.into_iter() {
-        match val {
-            Value::String(string) => result.insert(key, string),
-            _ => {
-                return Err(QueryError::ProtocolError(
-                    "json keyspaces map does not contain strings",
-                ))
-            }
-        };
-    }
-
-    Ok(result)
-}
-
 fn strategy_from_string_map(
     mut strategy_map: HashMap<String, String>,
 ) -> Result<Strategy, QueryError> {
@@ -323,7 +306,7 @@ fn strategy_from_string_map(
         ))?;
 
     let strategy: Strategy = match strategy_name.as_str() {
-        "org.apache.cassandra.locator.SimpleStrategy" => {
+        "org.apache.cassandra.locator.SimpleStrategy" | "SimpleStrategy" => {
             let rep_factor_str: String =
                 strategy_map
                     .remove("replication_factor")
@@ -337,7 +320,7 @@ fn strategy_from_string_map(
 
             Strategy::SimpleStrategy { replication_factor }
         }
-        "org.apache.cassandra.locator.NetworkTopologyStrategy" => {
+        "org.apache.cassandra.locator.NetworkTopologyStrategy" | "NetworkTopologyStrategy" => {
             let mut datacenter_repfactors: HashMap<String, usize> =
                 HashMap::with_capacity(strategy_map.len());
 
@@ -354,7 +337,7 @@ fn strategy_from_string_map(
                 datacenter_repfactors,
             }
         }
-        "org.apache.cassandra.locator.LocalStrategy" => Strategy::LocalStrategy,
+        "org.apache.cassandra.locator.LocalStrategy" | "LocalStrategy" => Strategy::LocalStrategy,
         _ => Strategy::Other {
             name: strategy_name,
             data: strategy_map,
